/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.client.internal.DashboardClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

dojo.require("jazz.client.xhr");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

var SERVICE_ID = "com.ibm.team.dashboard.common.internal.service.IDashboardRestService";

/*
 * Provides static methods to call the dashboard REST service.
 */
com.ibm.team.dashboard.web.client.internal.DashboardClient = {
	
	/*
	 * Creates and asynchronously returns a new dashboard for the given scope and
	 * process area.
	 */
	/* void */ createDashboard: function(/* string */ projectAreaItemId, /* number */ scope, /* string */ scopeItemId, /* string */ templateId, /* boolean */ save, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {scope: scope, scopeItemId: scopeItemId, save: save};
		if (projectAreaItemId) {
			params.projectAreaItemId = projectAreaItemId;
		}
		if (templateId) {
			params.templateId = templateId;
		}
		this._invoke("postCreateDashboard", params, false, null, srh, tracker);
	},
	
	/*
	 * Deletes the given dashboard.
	 */
	/* void */ deleteDashboard: function(/* string */ itemId, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {itemId: itemId};
		this._invoke("postDeleteDashboard", params, false, null, srh, tracker);
	},

	/*
	 * Deletes the given dashboard contributor info.
	 */
	/* void */ deleteDashboardContributorInfo: function(/* string */ itemId, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {itemId: itemId};
		this._invoke("postDeleteDashboardContributorInfo", params, false, null, srh, tracker);
	},
	
	/*
	 * Generates a new UUID and asynchronously returns it as a string to the handler.
	 */
	/* void */ generateUUID: function(/* int */ count, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		this._invoke("postGenerateUUID", {count: count || 1}, false, null, srh, tracker);
	},

	/*
	 * Fetches a dashboard by item id, user id, team name, or none of these (project).
	 */
	/* void */ getDashboard: function(/* string */ itemId, /* string */ teamName, /* string */ projectName, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {};
		if (projectName) {
			params.projectName = projectName;
		}
		if (itemId) {
			params.itemId = itemId;
		}
		if (teamName) {
			params.teamName = teamName;
		}
		this._invoke("getDashboard", params, false, null, srh, tracker);
	},
	
	/*
	 * Fetches the URL of the default dashboard to show, for the initial redirect.
	 */
	/* void */ getDefaultDashboardHref: function(/* string */ projectName, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {};
		if (projectName) {
			params.projectName = projectName;
		}
		this._invoke("getDefaultDashboardHref", params, false, null, srh, tracker);
	},
	
	/*
	 * Fetches the name of the requested item.
	 */
	/* string */ getItemName: function(/* string */ scope, /* string */ itemId, /* string */ repositoryRoot, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {scope: scope, itemId: itemId};
		this._invoke("getItemName", params, false, repositoryRoot, srh, tracker);
	},
	
	/*
	 * Retrieves the information to create the left nav for the current user.
	 */
	/* void */ getLeftNavInfo: function(/* string */ projectAreaName, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {};
		if (projectAreaName) {
			params.projectAreaName = projectAreaName;
		}
		this._invoke("getLeftNavInfo", params, false, null, srh, tracker);
	},
	
	/*
	 * Retrieves the information about the named templates.
	 * NOTE: This method is used by Customize viewlet. Refactor if needed (DG).
	 */
	/* void */ getNamedTemplatesInfo: function(/* string */ projectAreaName, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {projectAreaName: projectAreaName};
		this._invoke("getNamedTemplatesInfo", params, false, null, srh, tracker);
	},	
	
	/*
	 * Fetches a list of users related to the given user.
	 */
	/* void */ getRelatedUsers: function(/* string */ projectAreaItemId, /* string */ userId, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = { projectAreaItemId: projectAreaItemId, userId: userId };
		this._invoke("getRelatedUsers", params, false, null, srh, tracker);
	},

	/*
	 * Retrieves the information to create the viewlet settings area for the given user.
	 */
	/* void */ getSettingsAreaInfo: function(/* string */ userId, /* number */ scope, /* string */ scopeItemId, /* string */ currentSelectedContributorItemId,/* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {userId: userId, scope: scope, scopeItemId: scopeItemId, currentSelectedContributorItemId: currentSelectedContributorItemId};
		this._invoke("getSettingsAreaInfo", params, false, null, srh, tracker);
	},

	/*
	 * Retrieves the information to create the viewlet settings area for the given user.
	 */
	/* void */ getSharedDashboards: function(/* string */ projectAreaItemId, /* string */ filter, /* int */ startIndex, /* int */ maxResults, /* string */ token, /* string */ orderBy, /* boolean */ orderByAscending, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {projectAreaItemId: projectAreaItemId, startIndex: startIndex, maxResults: maxResults, orderByAscending: orderByAscending};
		if (filter) {
			params.filter = filter;
		}
		if (token) {
			params.token = token;
		}
		if (orderBy) {
			params.orderBy = orderBy;
		}
		this._invoke("getSharedDashboards", params, false, null, srh, tracker);
	},
	
	/*
	 * Retrieves the version of the dashboard framework, e.g. "1.0.0.1".
	 */
	/* void */ getVersion: function(/* string */ repositoryRoot, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		this._invoke("getVersion", {}, false, repositoryRoot, srh, tracker);
	},
	
	/*
	 * Retrieves the given viewlet definitions.
	 */
	/* void */ getViewletDefinitions: function(/* string[] */ definitionIds, /* string */ repositoryRoot, /* boolean */ passive, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {definitionIds: definitionIds};
		this._invoke("getViewletDefinitions", params, passive, repositoryRoot, srh, tracker);
	},
	
	/*
	 * Retrieves the hierarchy of categorized viewlet types.
	 */
	/* ServiceRequest */ getViewletHierarchy: function(/* string */ projectAreaName, /* boolean */ remoteOnly, /* string */ scope, /* string */ scopeItemId, /* string */ repositoryRoot, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {};
		if (projectAreaName) {
			params = {projectAreaName: projectAreaName, scope: scope, scopeItemId: scopeItemId};
		}
		if (remoteOnly) {
			params.remoteOnly = true;
		}
		return this._invoke("getViewletHierarchy", params, false, repositoryRoot, srh, tracker);
	},
	
	/*
	 * Migrates the given viewlet data and returns the migrated data.
	 */
	/* void */ migrateViewlet: function(/* Viewlet */ viewlet, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {json: dojo.toJson(viewlet)};
		this._invoke("postMigrateViewlet", params, false, null, srh, tracker);
	},

	/*
	 * Establishes or renews a soft lock on the given dashboard. Locks provide a
	 * warning to prevent several users from overwriting each others' changes.
	 */
	/* void */ lockDashboard: function(/* string */ itemId, /* string */ modified, /* boolean */ override, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {itemId: itemId, modified: modified};
		if (override) {
			params.override = true;
		}
		this._invoke("postLockDashboard", params, false, null, srh, tracker);
	},
	
	/*
	 * Saves the given dashboard.
	 */
	/* void */ saveDashboard: function(/* Dashboard */ dashboard, /* boolean */ override, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {dashboard: dojo.toJson(dashboard), override: override};
		this._invoke("postSaveDashboard", params, false, null, srh, tracker);
	},
	
	/*
	 * Saves the given dashboard contributor info.
	 */
	/* void */ saveDashboardContributorInfo: function(/* DashboardContributorInfo */ dashboardContributorInfo, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {dashboardContributorInfo: dojo.toJson(dashboardContributorInfo)};
		this._invoke("postSaveDashboardContributorInfo", params, false, null, srh, tracker);
	},

	/*
	 * Searches for dashboards and asynchronously returns a SearchResultsDTO
	 * containing the results.
	 */
	/* void */ search: function(/* string */ query, /* string */ projectAreaItemId, /* boolean */ includeTeams, /* boolean */ includeShared, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {query: query, projectAreaItemId: projectAreaItemId, includeTeams: includeTeams, includeShared: includeShared};
		this._invoke("postSearch", params, false, null, srh, tracker);
	},

	/*
	 * Releases the lock on the given dashboard.
	 */
	/* void */ unlockDashboard: function(/* string */ itemId, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var params = {itemId: itemId};
		this._invoke("postUnlockDashboard", params, false, null, srh, tracker);
	},

	/*
	 * Invokes a dashboard service call.
	 */
	/* ServiceRequest */ _invoke: function(/* string */ method, /* object */ params, /* boolean */ passive, /* string */ repositoryRoot, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var req = new ServiceRequest(SERVICE_ID, method, params);
		if (tracker) {
			tracker.register(req, srh);
		}
		var dataServerRoot = dojo.getObject("net.jazz.ajax._dataServerRoot");
		var xhr = jazz.client.xhr;
		if (repositoryRoot) {
			// temporarily set _dataServerRoot to remote server for request
			net.jazz.ajax._dataServerRoot = repositoryRoot;
			
			// temporarily hijack jazz.client for request
			jazz.client.xhr = function(method, args) {
				// reset dataServerRoot and xhr back immediately
				net.jazz.ajax._dataServerRoot = dataServerRoot;
				jazz.client.xhr = xhr;

				// skip oauth for passive requests
				if (passive) {
					args.skipInterceptors = ["jazz.oauth"];
				}
				return xhr.apply(this, arguments);
			};
		}
		TeamServerClient.invokeService(req, srh);
		return req;
	}
};

})();
