/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ViewletContainer");

dojo.require("dojo.i18n"); 
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.IFrameContainer");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.ScopePreferenceProvider");
dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.SettingsArea");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.DiscoveryRegistry");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ErrorReport");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.core.loader");
dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.styledBox.TrimStyles");
dojo.require("jazz.util.html");
dojo.require("jazz.util.ResizeNotifier");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

/*
 * Aliases
 */
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DiscoveryRegistry = com.ibm.team.dashboard.web.ui.internal.util.DiscoveryRegistry;
var ErrorReport = com.ibm.team.dashboard.web.ui.internal.util.ErrorReport;
var HTML = jazz.util.html;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var IFrameContainer = com.ibm.team.dashboard.web.ui.internal.util.IFrameContainer;
var ItemNameRegistry = com.ibm.team.dashboard.web.ui.internal.util.ItemNameRegistry;
var loader = jazz.core.loader;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
var ResizeNotifier = jazz.util.ResizeNotifier;
var ScopePreferenceProvider = com.ibm.team.dashboard.web.ui.internal.settings.ScopePreferenceProvider;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceTracker = com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker;
var SettingsArea = com.ibm.team.dashboard.web.ui.internal.settings.SettingsArea;
var TrimStyles = jazz.ui.styledBox.TrimStyles;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var ViewletRegistry = com.ibm.team.dashboard.web.ui.internal.util.ViewletRegistry;

var SERVICE_IMG = "com.ibm.team.dashboard.common.internal.service.IDashboardImageService";

var STATE_LOADING = 0;
var STATE_LOGIN = 1;
var STATE_ACTIVE = 2;
var STATE_ERROR = 3;

var messages;

/*
 * (static) Observables to listen for remote viewlet log-in, to allow viewlets
 * from the same origin to load as soon as one logs in.
 */
var _loginTrackersByOrigin;

/*
 * The viewlet container that houses the viewlet trim, titlebar, etc.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ViewletContainer",
	[jazz.ui.StyledBox, Observable], {
	
	/*
	 * Variables that must be passed into constructor
	 */
	dashboardBundle: null, 		//required
	viewletHelper: null,		//required
	viewletPage: null,			//required
	
	trim: TrimStyles.BLUE,		//default to BLUE
	collapsible: true,
	editing: null,
	truncateTitle: false,
	menuItems: new Array(),
	
	/*
	 * Internal variables - not API
	 */
	_viewletDefinition: null,
	_viewletContributorInfo: null,
	_columnIndex: null,
    _serviceTracker: null,
	_errorReport: null,
	_state: STATE_LOADING,
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/ViewletContainer.html"),

	constructor: function(data) {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	
	postCreate: function() {
		// show initial loading state
		this._initializeContributorInfo();
		this._columns = this.viewletPage.columns;
		this.primaryTitle = this.viewletHelper.title;
		this.iconPath = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/spinner.gif");
		this.collapsed = this._viewletContributorInfo.collapsed ? true : false;
		this.trim = this.viewletHelper.viewletTrim;
		this.showBackground = this.viewletHelper.showBackground;
		this.inherited(arguments);
		this._zoomImg.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/zoom-in.gif");
		this.dashboardContainer = this.viewletPage.dashboardContainer;
		
		if (!this._isLocal()) {
			this._remoteIcon = document.createElement("div");
			this._remoteIcon.className = "remote-icon remote-icon-disconnected";
			var origin = this.viewletHelper.origin;
			origin = origin.substring(0,origin.lastIndexOf("/"));
			this._shortOrigin = origin;
			this._remoteIcon.title = dojo.string.substitute(messages.disconnected,[messages.remoteServer,origin]);
			this._headerButtons.insertBefore(this._remoteIcon,this._headerButtons.firstChild);
			this.dashboardContainer._showFirebugWarning();
		}
		
		// yield
		this._yieldTimeout = setTimeout(dojo.hitch(this, this._loadViewlet, true), 0);
	},
	/*
	 * Loading stage 1
	 * Locate viewlet origin server
	 */
	_loadViewlet: function(/* boolean */ passive) {
		// 94229: clearTimeout doesn't always work on IE7
		if (this._isDestroyed) {
			return;
		}
		delete this._yieldTimeout;
		
		if (this._isLocal()) {
			// local viewlets use local data server
			this._loadViewlet2(false);
		} else {
			// remote viewlet; load origin server info
			var serviceObj = {
				_self: this,
				_passive: passive,
				success: function(/* object */ serverData) {
					this._self._serverData = serverData;
					this._self._loadViewlet2(this._passive);
					DiscoveryRegistry.getFriend(this._self.viewletHelper.origin, dojo.hitch(this._self, this._self._updateRemoteTooltip));
				},
				failure: function(/* Error */ error) {
					DiscoveryRegistry.getFriend(this._self.viewletHelper.origin, dojo.hitch(this._self, this._self._updateRemoteTooltip));
					if (error && error.status == 401) {
						this._self._showLogin();
					}
					else if (error && error.status == 504) {
						this._self._showError(error.message, messages.remoteConnectionError);
					}
					else {
						this._self._showError(error);
					}
				}
			};
			DiscoveryRegistry.getServerData(this.viewletHelper.origin, passive, dojo.hitch(serviceObj, serviceObj.success), dojo.hitch(serviceObj, serviceObj.failure));
		}
	},

	/*
	 * Loading stage 2
	 * Fetch viewlet definition
	 */
	_loadViewlet2: function(/* boolean */ passive) {
		var serviceObj = {
			_self: this,
			_onResponse: function() {
				// notify any login-waiting viewlets to load
				if (!this._self._isLocal() && this._self._loginCallback) {
					this._self._removeLoginTracker();
					_loginTrackersByOrigin[this._self.viewletHelper.origin].propertyChanged("login");
				}
			},
			success: function(/* ViewletDefinitionDTO[] */ definitions) {
				this._onResponse();
				if (definitions && definitions.length > 0) {
					this._self._viewletDefinition = definitions[0];
					this._self._loadViewlet3();
				}
				else {
					// couldn't find definition
					var msg = dojo.string.substitute(messages.viewletDefNotFound, [this._self.viewletHelper.definitionId]);
					this._self._showError(msg);
				}
			},
			failure: function(/* Error */ error) {
				if (error && error.status == 401) {
					this._self._showLogin();
				}
				else {
					this._onResponse();
					this._self._showError(error);
				}
			}
		};
		var dataServerRoot = this._isLocal() ? null : this._serverData.viewletServiceRoot;
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		ViewletRegistry.getDefinition(this.viewletHelper.definitionId, dataServerRoot, passive, srh, this._getServiceTracker());
	},
	
	/*
	 * Loading stage 3
	 * Fetch scope item name
	 */
	_loadViewlet3: function() {
		var scopeItem = this.viewletHelper.scopeItem;
		if (scopeItem) {
			var itemId = scopeItem.itemId;
			var name = ItemNameRegistry.getName(itemId);
			if (!name) {
				// don't have it; fetch it
				var serviceObj = {
					_this: this,
					_itemId: itemId,
					success: function(/* string */ name) {
						// add to registry and scope item
						ItemNameRegistry.addName({itemId: this._itemId, name: name});
						scopeItem.name = name;
						this._this._loadViewlet4();
					},
					failure: function(/* Error */ error) {
						// skip name and continue without it
						this._this._loadViewlet4();
					}
				};
				var dataServerRoot = this._isLocal() ? null : this._serverData.viewletServiceRoot;
				var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
				DashboardClient.getItemName(this.viewletHelper.scope, itemId, dataServerRoot, srh, this._getServiceTracker());
			}
			else {
				// already have it; set and continue
				scopeItem.name = name;
				this._loadViewlet4();
			}
		}
		else {
			this._loadViewlet4();
		}
	},
	
	/*
	 * Loading stage 4
	 * Initialize viewlet container, migrate data
	 */
	_loadViewlet4: function() {
		this.truncateTitle = true;
		this.setTitle(this.primaryTitle);
		this.setSecondaryTitle("");
		
		var icon = this.viewletHelper.icon ? this.viewletHelper.icon : this._viewletDefinition.icon;
		var isAbsolute = (icon.indexOf("://") != -1);
		var params = {definitionId: this._viewletDefinition.id};
		params[isAbsolute ? "url" : "path"] = icon;
		this.iconPath = Util.getServiceURL(SERVICE_IMG, null, params, this._isLocal() ? null : this._serverData.viewletServiceRoot);
		this.setIcon(this.iconPath);

		if (!this._isLocal()) {
			// remote viewlet may have been updated, but its data may be old.
			// check if migration is needed
			var dataVersion = this.viewletHelper.version;
			var viewletVersion = this._viewletDefinition.version;
			if ((dataVersion || viewletVersion) && (dataVersion != viewletVersion)) {
				// version mismatch; ask remote server to migrate
				var serviceObj = {
					_self: this,
					success: function(/* ResponseDTO */ response) {
						if (response.status == "ERROR") {
							this._self._showError(response.msg);
						}
						else if (!response.data) {
							// migration handler deleted viewlet
							this._self._showMessage(messages.viewletDeleted);
						}
						else {
							dojo.mixin(this._self.viewletHelper, response.data);
							this._self._loadViewlet5();
						}
					},
					failure: function(/* Error */ error) {
						this._self._showError(error);
					}
				};
				var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
				DashboardClient.migrateViewlet(this.viewletHelper, srh, this._getServiceTracker());
				return;
			}
		}
		this._loadViewlet5();
	},
	
	/*
	 * Loading stage 5
	 * Load & instantiate viewlet widget
	 */
	_loadViewlet5: function() {
		if (this._isLocal()) {
			// load code & css
			loader.load_async(this._viewletDefinition.widget, dojo.hitch(this, function() {
				if (!this._isDestroyed) {
					try {
						// create and add the viewlet (the content)
						var viewletConstructor = eval(this._viewletDefinition.widget);
						this._viewlet = new viewletConstructor({
							dashboardItem: this.dashboardBundle.dashboard,
							dashboardBundle: this.dashboardBundle,
							definition: this._viewletDefinition,
							helper: this.viewletHelper,
							container: this
						});
						this._bodyContainer.replaceChild(this._viewlet.domNode, this._loadingDiv);
						this._loadViewlet6();
					} 
					catch (/* Error */ error) {
						this._showError(error);
					}
				}
			}));
		}
		else {
			// find the page container, which marks the iframe "bust out" bounds
			var pageNode = this.domNode;
			while (true) {
				if (pageNode.id == "content-cell" || !pageNode.parentNode) {
					break;
				}
				pageNode = pageNode.parentNode;
			}
			
			// remote viewlet; iframe it
			this._iframeContainer = new IFrameContainer({
				active: !this.collapsed,
				className: this._viewletDefinition.widget,
				webUIRoot: this._serverData.viewletWebUIRoot,
				serviceRoot: this._serverData.viewletServiceRoot,
				args: {
					dashboardItem: this.dashboardBundle.dashboard,
					dashboardBundle: this.dashboardBundle,
					definition: this._viewletDefinition,
					helper: this.viewletHelper,
					container: this
				},
				onLoad: dojo.hitch(this, function(/* Widget */ widget) {
					// 94229: clearTimeout doesn't always work on IE7
					if (!this._isDestroyed) {
						this._viewlet = widget;
						this._loadViewlet6();
					}
				}),
				pageNode: pageNode
			});
			this._bodyContainer.replaceChild(this._iframeContainer.domNode, this._bodyContainer.firstChild);
		}
		//Register event handlers to show zoom link if viewlet is zoomable
		if (this._viewletDefinition.zoomable) {
			this._zoom.style.display = "block";
			this.connect(this._middleSection, 'onmouseover', this._showZoom);
			this.connect(this._middleSection, 'onmouseout', this._hideZoom);
			this.connect(this._zoom, 'onclick', this.toggleZoom);
		}
	},
	
	/*
	 * Loading stage 6
	 * Finish configuring viewlet, perform initial refresh
	 */
	_loadViewlet6: function() {
		this.setEditMode(this.editing);
		
		//setup auto-refresh
		if (this._viewletDefinition.refreshInterval > 0) {
			this.connect(this._viewlet, "refresh", "onRefresh");
		}
		// clear the viewlet menu in case this is a re-load
		delete this._contributedMenuItems;
		//allow viewlets to initialize themselves before the first refresh
		this._viewlet.init();
		//inform viewlet that it is active
		this.setActive(!this.collapsed);
		this._state = STATE_ACTIVE;
		//trigger initial refresh
		this.refreshViewlet();
	},
	_showLogin: function() {
		// get friend info so we can show its name in the login section
		DiscoveryRegistry.getFriend(this.viewletHelper.origin, dojo.hitch(this, this._showLogin2), dojo.hitch(this, this._showError));
	},
	_showLogin2: function(/* object */ friend) {
		// show login required icon
		this.iconPath = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/login.gif");
		this.setIcon(this.iconPath);
		this.setSecondaryTitle(messages.viewletNotLoggedIn);
		
		// listen for log-ins for this origin
		this._addLoginTracker();
		
		// show log-in section
		var div = document.createElement("div");
		div.className = "login";
		div.innerHTML = "<div class=\"message\">" + dojo.string.substitute(messages.viewletLogin, [friend.title]) + "</div>";
		
		// add log-in link
		var linkDiv = document.createElement("div");
		linkDiv.className = "link";
		var a = document.createElement("a");
		a.href = "#";
		a.appendChild(document.createTextNode(messages.viewletLogin2));
		this.connect(a, "onclick", "_onLogin");
		linkDiv.appendChild(a);		
		div.appendChild(linkDiv);
		
		this._bodyContainer.replaceChild(div, this._bodyContainer.firstChild);
		this._state = STATE_LOGIN;
	},
	_updateRemoteTooltip: function(obj) {
		if (!obj) return;
		if (this._serverData) {
			this._remoteIcon.title = dojo.string.substitute(messages.connected,[obj.title, this._shortOrigin]);
			this._remoteIcon.className = "remote-icon remote-icon-connected";
		} else {
			this._remoteIcon.title = dojo.string.substitute(messages.disconnected,[obj.title, this._shortOrigin]);
			this._remoteIcon.className = "remote-icon remote-icon-disconnected";
		}
	},
	_showMessage: function(/* string */ msg) {
		var div = document.createElement("div");
		div.className = "msg";
		div.appendChild(document.createTextNode(msg));
		this._bodyContainer.replaceChild(div, this._bodyContainer.firstChild);
		this._toggleMenu.parentNode.removeChild(this._toggleMenu);
		this._state = STATE_ERROR;
	},
	_addLoginTracker: function() {
		if (!_loginTrackersByOrigin) {
			_loginTrackersByOrigin = new Object();
		}
		var tracker = _loginTrackersByOrigin[this.viewletHelper.origin];
		if (!tracker) {
			tracker = _loginTrackersByOrigin[this.viewletHelper.origin] = new Observable();
		}
		this._loginCallback = dojo.hitch(this, function() {
			this._removeLoginTracker();
			this._loadViewlet(false);
		});
		tracker.addListener(this._loginCallback, "login");
	},
	_removeLoginTracker: function() {
		if (this.viewletHelper.origin && _loginTrackersByOrigin && _loginTrackersByOrigin[this.viewletHelper.origin]) {
			_loginTrackersByOrigin[this.viewletHelper.origin].removeListener(this._loginCallback);
			delete this._loginCallback;
		}
	},
	_onLogin: function(/* Event */ event) {
		if (event) {
			dojo.stopEvent(event);
		}
		this._loadViewlet(false);
	},
	_isLocal: function() {
		return Util.isLocal(this.viewletHelper);
	},
	_showError: function(/* Error/string */ error, /* string */ msg) {
		// error icon
		this.iconPath = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/error.gif");
		this.setIcon(this.iconPath);
		
		// error message w/show details
		var args = {error: error};
		if (msg) {
			args.msg = msg;
		}
		this._errorReport = new ErrorReport(args);
		this._bodyContainer.replaceChild(this._errorReport.domNode, this._bodyContainer.firstChild);
		this._toggleMenu.parentNode.removeChild(this._toggleMenu);
		this._state = STATE_ERROR;
	},
	_initializeContributorInfo: function() {
		var viewletInfo = this.dashboardBundle.contributorInfo.viewletInfo;
		for (var i=0; i < viewletInfo.length; i++) {
			if (viewletInfo[i].viewletId == this.viewletHelper.id) {
				// already exists; grab it
				this._viewletContributorInfo = viewletInfo[i];
				break;
			}
		}
		if (!this._viewletContributorInfo) {
			// create an empty one
			this._viewletContributorInfo = { viewletId: this.viewletHelper.id };
			viewletInfo.push(this._viewletContributorInfo);
		}
	},
	/*
	 * Returns the CustomPreferenceProvider implementation class for the special
	 * viewlet scope preference. In the iframed remote viewlet case, this is a
	 * class from inside the iframe.
	 */
	getScopePreferenceProviderClass: function() {
		if (this._iframeContainer) {
			return this._iframeContainer.getScopePreferenceProviderClass();
		}
		return ScopePreferenceProvider;
	},
	/*
	 * Called whenever the viewlet's refresh method is called (via
	 * this.connect). Resets the auto-refresh timer.
	 */
	onRefresh: function() {
		if (this._refreshTimeout) {
			clearTimeout(this._refreshTimeout);
			delete this._refreshTimeout;
		}
		// recall the last refresh time so that we can properly suspend and resume
		// auto-refresh without disrupting the normal schedule
		this._lastRefreshTime = new Date().getTime();
		this._refreshTimeout = setTimeout(dojo.hitch(this, this._handleAutoRefresh), this._viewletDefinition.refreshInterval * 1000);
	},
	setTrim: function(trim) {
		/* Convert trim values into StyledBox style params */
		if (trim == 1) trim = TrimStyles.GRAY;
		else if (trim == 2) trim = TrimStyles.GOLD;
		else if (trim == 3) trim = TrimStyles.GREEN;
		else if (trim == 4) trim = TrimStyles.TEAL;
		else if (trim == 5) trim = TrimStyles.PURPLE;
		else if (trim == 6) trim = TrimStyles.ORANGE;
		else if (trim == 7) trim = TrimStyles.TRIMLESS;
		else trim = TrimStyles.BLUE;
		this.inherited(arguments);
	},
	_truncate: function(force) {
		if (!this.primaryTitle || !this.truncateTitle) return;
		// 94229: clearTimeout doesn't always work on IE7
		if (this._isDestroyed) {
			return;
		}
		this.truncateTimeout = null;
		if (this._viewletDefinition && this._viewletDefinition.titleAsHyperlink) {
			var node = this._headerPrimary.firstChild;
		} else {
			var node = this._headerPrimary;
		}
		// force is used when the title has been changed and the truncate algorithm needs to be run
		// even if the clientWidth is the same
		// force == true is used since this function is called from many places, some of which
		// pass in parameters
		if (force == true || (this._titleIsTruncated && this._header.clientWidth > this._prevWidth)) {
			node.innerHTML = HTML.escape(this.primaryTitle);
			this._titleIsTruncated = false;
			node.title = "";
		}
		if (this._headerText.clientHeight > this._header.clientHeight + 5) { // 5px buffer for browser em height calculation anomalies
			var title = node.innerHTML;
			while (this._headerText.clientHeight > this._header.clientHeight + 5) { // 5px buffer for browser em height calculation anomalies
				if (this._headerText.clientHeight > this._header.clientHeight * 2) {
					title = title.substring(0,title.length/2);
				} else {
					title = title.substring(0,title.length-2);
				}
				while (title.lastIndexOf(" ") == title.length-1)
					title = title.substring(0,title.length-1);
				node.innerHTML = title + "...";
				if (node.innerHTML.length <= 6) break;
			}
			node.title = this.primaryTitle;
			this._titleIsTruncated = true;
			this._prevWidth = this._header.clientWidth;
		}
	},
	setEditMode: function(editing) {
		this.editing = editing;
		this._viewlet.setDashboardEditing(editing);
		if (this._menu) {
			this._menu.destroy();
			this._menu = null;
		}
		if(editing) {
			// add listeners
			if (!this.editModeListeners) {
				this.editModeListeners = new Array();
			}
			// create buttons
			if (!this._edit && this._viewletDefinition && this._viewletDefinition.editable) {
				this._edit = document.createElement("a");
				this._edit.href="#";
				this._edit.title = messages.editContents;
				this._edit.className = "edit-contents";
				var editImg = document.createElement("img");
				editImg.src = dojo.moduleUrl('com.ibm.team.dashboard.web', "graphics/content/images/edit-contents.gif");
				editImg.alt = messages.edit;
				this._edit.appendChild(editImg);
				var beforeNode = this._headerButtons.lastChild;
				if (beforeNode.nodeType == 3) beforeNode = beforeNode.previousSibling;
				this._headerButtons.insertBefore(this._edit, beforeNode);
				this.editModeListeners.push(this.connect(this._edit, 'onclick', this.startEditing));
			}
			this.setClosable(true);
		} else {
			while (this.editModeListeners && this.editModeListeners.length >0){
				this.disconnect(this.editModeListeners.pop());
			}
			if (this._edit) {
				this._edit.parentNode.removeChild(this._edit);
				this._edit = null;
			}
			this.setClosable(false);
			if (this._settingsOpen) {
				this.toggleSettings();
			}
			if (this._viewlet.isEditing()) {
				this.stopEditing();
			}
		}
	},
	/*
	 * Creates a copy of the viewlet, with a new id, and updates the
	 * dashboard state.
	 */
	cloneViewlet: function(event) {
		if (this.isZoomed) this.toggleZoom (false, true);

		var columnHelper = this._columns[this._columnIndex]._columnHelper;
		var viewlets = columnHelper.viewlets;
		for (var i=0;i<viewlets.length;i++) {
			if (viewlets[i].id == this.viewletHelper.id) {
				this.viewletPage.addViewletFromHelper(this.viewletHelper, this._columnIndex, i);
				break;
			}
		}
	},
	copyViewlet: function() {
		var viewletHelper = dojo.clone(this.viewletHelper);
		if (typeof(viewletHelper.scope) == 'undefined') {
			viewletHelper.scope = this.dashboardContainer.dashboardItem.scope;
			viewletHelper.scopeItem = this.dashboardContainer.dashboardItem.scopeItem;
		}
		try {
			globalStorage[location.hostname]["com-ibm-team-dashboard-clipboard"] = dojo.toJson(viewletHelper);
		} catch (e) {
			alert(messages.featureNotSupported);
		}
	},
	/*
	 * Called when the user clicks on the viewlet container's close button.
	 */
	close: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		if (confirm(messages.closeViewlet)) {
			if (this.isZoomed) this.toggleZoom();
			var args = {node: this.domNode, duration: 250};
			var fadeOut = dojo.fadeOut(args);
			var wipeOut = dojo.fx.wipeOut(args);
			this.connect(wipeOut, "onEnd", this._onCloseAnimationEnd);
			dojo.fx.chain([fadeOut,wipeOut]).play();
		}
	},
	_onCloseAnimationEnd: function() {
		this._columns[this._columnIndex].remove(this.index);
	},
	getHeaderLink: function(event) {
		if (this._viewlet) {
			var url = this._viewlet.getTitleHyperlinkURL();
			if (url) {
				this._headerPrimary.firstChild.href=url;
			} else {
				this._headerPrimary.firstChild.removeAttribute('href');
			}
		}
	},
	isVisible: function() {
		return this.viewletPage.isVisible();
	},
	setTitle: function(/* string */ title, /* boolean */ persist) {
		var txt = document.createTextNode(title);
		if (this._viewletDefinition && this._viewletDefinition.titleAsHyperlink) {
			if (this._headerPrimary.firstChild && this._headerPrimary.firstChild.firstChild) {
				this._headerPrimary.firstChild.replaceChild(txt, this._headerPrimary.firstChild.firstChild);
			}
			else {
				var link = document.createElement("a");
				link.className = "title-link";
				link.appendChild(txt);
				if (this._headerPrimary.firstChild) {
					this._headerPrimary.replaceChild(link, this._headerPrimary.firstChild);	
				} else {
					this._headerPrimary.appendChild(link);
				}
				this.headerLinkEvent = this.connect(link,"onmouseover",this.getHeaderLink);
			}
		} else {
			if (this._headerPrimary.firstChild) {
				this._headerPrimary.replaceChild(txt,this._headerPrimary.firstChild);	
			} else {
				this._headerPrimary.appendChild(txt);
			}
		}
		this.primaryTitle = title;
		if (persist) {
			this.viewletHelper.title = title;
			this.propertyChanged(Constants.PROPERTY_DASHBOARD);
		}
		this._setupTruncate();
	},
	getSecondaryTitle: function() {
		return this._headerSecondary.innerHTML;
	},
	setActive: function(/* boolean */ value) {
		if (this._iframeContainer) {
			this._iframeContainer.setActive(value);
		}
		if (this._viewlet) {
			this._viewlet.setActive(value);
		}
	},
	setAutoRefresh: function(/* boolean */ value) {
		if (this._viewletDefinition.refreshInterval > 0) {
			if (value && !this._refreshTimeout) {
				var timeSinceLastRefresh = new Date().getTime() - this._lastRefreshTime;
				var timeUntilNextRefresh = (this._viewletDefinition.refreshInterval * 1000) - timeSinceLastRefresh;
				// always wait at least 5 sec before an auto-refresh to avoid disruptions
				if (timeUntilNextRefresh < 5000) {
					timeUntilNextRefresh = 5000;
				}
				// resume auto-refresh
				this._refreshTimeout = setTimeout(dojo.hitch(this._viewlet, "refresh"), timeUntilNextRefresh);
			}
			else if (!value && this._refreshTimeout) {
				// suspend auto-refresh
				if (this._refreshTimeout) {
					clearTimeout(this._refreshTimeout);
					delete this._refreshTimeout;
				}
			}
		}
	},
	startEditing: function(event) {
		if (event) dojo.stopEvent(event);
		if (!this._viewlet.isEditing()) {
			var height = this._contentWrapper.clientHeight;
			this.setAutoRefresh(false);
			if (this.collapsed) {
				this.toggleCollapsed();
			}
			if(!this.editingButtons) this.editingButtons = [];

			if(!this.isZoomed)
				this._contentWrapper.style.height = this._contentWrapper.clientHeight + "px";
			
			this._editButtonArea.style.display = "block";
			this.editingButtons.push(this.connect(this._saveEdits,"onclick",this.stopEditing));
			this.editingButtons.push(this.connect(this._cancelEdits,"onclick",this.stopEditing));
			
			dojo.addClass(this.domNode, "com-ibm-team-dashboard-viewlet-editing");
			
			this._viewlet.getViewletState().startEditing();
			this._viewlet.setEditing(true);
			if (!this.isZoomed) {
				this._contentWrapper.style.height = height + "px";
				var anim = dojo.animateProperty({node: this._contentWrapper, duration: 250, 
					properties: 
						{height: {start: height, end: this._bodyContainer.clientHeight + this._editButtonArea.clientHeight + this._settingsContainer.clientHeight, unit: "px"}
					}});	
				anim.play();
				this.connect(anim,"onEnd",function() {
					this._contentWrapper.style.height = "";
					dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
				});	
			}
			this._addHoverClass(null, true);
		}
	},
	stopEditing: function(e) {
		var height = this._contentWrapper.clientHeight;
		//Collapsing viewlet while in edit mode calls this method
		var saving = false;
		//if the event handler exists, then a button was explicitly pressed
		if (!e) {
			if(!confirm(dojo.string.substitute(messages.stopEditingConfirm, [this.viewletHelper.title]))){
				return false;
			}
		} else {
			var saving = e.target == this._saveEdits;
		}
		if (saving) {
			// "Save" button was clicked
			// Validate edited content
			var validContent = this._viewlet.validateEditedContent();
			// Check if the content is valid
			if (validContent == false) {
				// Content is not valid.  Stay in edit mode
				return;
			}
			// Content is valid.  Leave edit mode
			// Notify viewlet that edited contents is about to be saved
			this._viewlet.preSaveState();
		}
		this._editButtonArea.style.display = "none";
		dojo.removeClass(this.domNode, "com-ibm-team-dashboard-viewlet-editing");
		this._viewlet.getViewletState().stopEditing(saving);
		this._viewlet.setEditing(false);
		while(this.editingButtons.length > 0) {
			this.disconnect(this.editingButtons.pop());
		}
		this.setAutoRefresh(true);
		
		if (!this.isZoomed) {
			this._contentWrapper.style.height = height + "px";
			var anim = dojo.animateProperty({node: this._contentWrapper, duration: 250, 
				properties: 
					{height: {start: height, end: this._bodyContainer.clientHeight + this._settingsContainer.clientHeight, unit: "px"}
				}});	
			anim.play();
			this.connect(anim,"onEnd",function() {
				this._contentWrapper.style.height = "";
				dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
			});	
		}
		this._removeHoverClass(null, true);
		if (!e) return true;
	},
	_toggleCollapse: function(event) {
		if (this._viewlet && this._viewlet.isEditing()) {
			if(!this.stopEditing()) {
				return;
			}
		}
		if (this.isZoomed) {
			this.toggleZoom();
		}
		if (this._settingsOpen) {
			this.toggleSettings();
		}
		this.setActive(this.collapsed);
		this.inherited(arguments);
	},
	_handleAutoRefresh: function() {
		// skip refresh unless viewlet is showing
		if (this.isVisible()) {
			this.refreshViewlet();
		}
		else {
			// try again next interval
			this._refreshTimeout = setTimeout(dojo.hitch(this, this._handleAutoRefresh), this._viewletDefinition.refreshInterval * 1000);
		}
	},
	refreshViewlet: function(/* boolean */ force) {
		if (this._state == STATE_LOGIN) {
			// try loading again, in case user logged in independently
			this._loadViewlet(true);
		}
		else {
			if (!force && this.isRefreshing)
				return;
			if (!force)
				this.isRefreshing = 1;
			else
				this.isRefreshing++;
			this._viewlet.refresh();
		}
	},
	_onCollapseAnimationEnd: function(){
		this.inherited(arguments);
		
		// update contributor info
		this._viewletContributorInfo.collapsed = this.collapsed;
		this.propertyChanged(Constants.PROPERTY_CONTRIBUTOR_INFO);
	},
	_removeHoverClass: function(event, force) {
		if (this._menuCloseOnBody || (this._viewlet && this._viewlet.isEditing()) || this._settingsOpen || this.isZoomed || (this.trim == TrimStyles.TRIMLESS && this.collapsed)) {
			return;
		}
		this.inherited(arguments);
	},
	toggleSettings: function(event) {
		if(!this.settingsWipeInAnimation || !this.settingsWipeOutAnimation) {
			var args = {node: this._settingsContainer, duration: 300};
			this.settingsWipeInAnimation = dojo.fx.wipeIn(args);
			this.connect(this.settingsWipeInAnimation,"onEnd",this._afterExpandSettings);	
			this.settingsWipeOutAnimation = dojo.fx.wipeOut(args);
			this.connect(this.settingsWipeOutAnimation,"onEnd",this._afterCollapseSettings);	
		}
		this.setAutoRefresh(this._settingsOpen);
		this._settingsOpen = !this._settingsOpen;
		if (!this._settingsOpen) {
			this._hideZoom();
			this.settingsWipeOutAnimation.play();
			this._removeHoverClass(null, true);
		} else {
			if (this.collapsed) {
				this.toggleCollapsed();
			}
			if (this._settingsContainer.childNodes.length > 0) {
				while (this._settingsContainer.firstChild) {
					this._settingsContainer.removeChild(this._settingsContainer.firstChild);
				}
			}
			this._settingsWidget = new SettingsArea({dashboardBundle: this.dashboardBundle, dashboardItem: this.dashboardBundle.dashboard, viewlet: this._viewlet, viewletHelper: this.viewletHelper, viewletDefinition: this._viewletDefinition, viewletContainer: this, parseOnLoad: true});
			this._settingsWidget.addListener(dojo.hitch(this, this._onChange), Constants.PROPERTY_DASHBOARD);
			this._settingsContainer.appendChild(this._settingsWidget.domNode);
			this._settingsContainer.style.display = "block";
			this._settingsContainer.style.height = "1px";
			this._settingsWidget.init();
		
			this.settingsWipeInAnimation.play();

			if (this.trim == TrimStyles.TRIMLESS && !this._headerVisible)
				this._toggleHeader();
			this._addHoverClass(null, true);
		}
		// Notify the viewlet that it's settings area was toggled
		this._viewlet.settingsAreaToggled(this._settingsOpen);
	},
	_afterCollapseSettings: function() {
		this._settingsWidget.destroy();
		this._settingsWidget = null;	
	},
	_afterExpandSettings: function() {
		dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		var coords = dojo.coords(this.domNode);
		if (this.domNode && this.domNode.scrollIntoView && coords.h + coords.y > dijit.getViewport().h) {
			this.domNode.scrollIntoView();
		}
	},
	toggleZoom: function(event, noAnimation) {
		if (event){
			if (this.zoomDiv && this.zoomDiv.style.display == "none") return;
			if (this.isZoomed && dojo.isDescendant(event.target, this.domNode) && event.target != this._zoom) return;
			dojo.stopEvent(event);
		}
		//do not toggle zoom when dashboard is not in focus
		if (!this.viewletPage.isActive) {
			return;
		}
		
		// Notify viewlet that a zoom operation is about to occur
		this._viewlet.preZoom();

		if (!this.isZoomed) {
			this._zooming = true;
			this._calcZoomVariables();

			this.dashboardContainer._showBlockerDiv();
			
			//create a placeholder for viewlet
			this.zoomPlaceholder = document.createElement("div");
			
			var offset = 0;
			if (this.trim == TrimStyles.TRIMLESS) {
				offset = 3;
			} 
			
			if (this.index && this.index == 0) {
				this.zoomPlaceholder.style.height = this.initialDimensions.h - offset + "px";
			} else {
				this.zoomPlaceholder.style.height = this.initialDimensions.h - offset - dojo.style(this.domNode,"marginBottom") + "px";
			}
			dojo.addClass(this.zoomPlaceholder, "com-ibm-team-dashboard-content-viewlet");
			this.domNode.parentNode.insertBefore(this.zoomPlaceholder, this.domNode);
						
			//create an absolutely positionned div and append viewlet
			this.zoomDiv = this.domNode;
			if (dojo.isIE) {
				document.body.appendChild(this.domNode);
				if (this._columnIndex === 0) {
					dojo.addClass(this.domNode, "column1");	
				}
			}
			dojo.addClass(this.zoomDiv, "com-ibm-team-dashboard-zoom-container");
			this.zoomDiv.style.position = "absolute";
			this.zoomDiv.style.zIndex = "999";
			
			this.dashboardContainer.zoomDiv = this.zoomDiv;
			this.dashboardContainer._updateZoomMargins = dojo.hitch(this,this._updateZoomMargins);
			
			//create and position viewlet at same place as it started, except absolutely positioned 
			dojo.marginBox(this.zoomDiv, {l: this.initialDimensions.l, t: this.initialDimensions.t, w: this.initialDimensions.w});
			
			//animate position and width
			if (!noAnimation) {
				if (this._columns.length == 1) {
					var anim = dojo.animateProperty({node: this.zoomDiv, duration: 250, 
					properties: 
						{height: {start: this.initialDimensions.h, end: this.endDimensions.h, unit: "px"},
						top: {end: this.endDimensions.t, unit: "px"}
						}});
				} else {
					var anim = dojo.animateProperty({node: this.zoomDiv, duration: 250, 
					properties: 
						{width: {start: this.initialDimensions.w, end: this.endDimensions.w, unit: "px"}, 
						height: {start: this.initialDimensions.h, end: this.endDimensions.h, unit: "px"},
						top: {end: this.endDimensions.t, unit: "px"},
						left: {end: this.endDimensions.l, unit: "px"}
						}});	
				}
				//animate height
				var anim2 = dojo.animateProperty({node: this._contentWrapper, duration: 250, properties: {height: {end: this.endDimensions.h - 24, unit: "px"}}});
					
				//run animations
				anim.play();
				anim2.play();	
			} else {
				this.zoomDiv.style.height = this.endDimensions.h + "px";
				this.zoomDiv.style.top = this.endDimensions.t + "px";
				if (this._columns.length != 1) {
					this.zoomDiv.style.width = this.endDimensions.w + "px";
					this.zoomDiv.style.left = this.endDimensions.l + "px";
				}
				this._contentWrapper.style.height = this.endDimensions.h - 24 + "px";
			}
			
			//change viewlet zoom icon
			dojo.removeClass(this._zoom, "zoom-in");
			dojo.addClass(this._zoom, "zoom-out");
			this._zoomImg.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/zoom-out.gif");
			
			this._contentWrapper.style.overflow = "auto"; // Must define this programatically to fix scrollbar bug in IE
			this._zoomWindowResize = ResizeNotifier.register({context: this, callback: this._updateZoomMargins});
			
			this.isZoomed = true;
			
			this._addHoverClass();
			
			if (!noAnimation) {
				this.connect(anim, "onEnd", this._onExpandEnd);	
			} else {
				this._onExpandEnd();
			}
			this._closeOnEsc = this.connect(document.body,"onkeypress", function(event) {
				if (event.keyCode == dojo.keys.ESCAPE) {
					this.toggleZoom();
				}
			})
			this._showZoom();
			this._header.focus();
		} else {
			this._zooming = true;
			if (!noAnimation) {
				//animate position and width
				if (this._columns.length == 1) {
					var anim = dojo.animateProperty({node: this.zoomDiv, duration: 250, 
					properties: 
						{height: {end: this.initialDimensions.h, unit: "px"},
						top: {end: this.initialDimensions.t, unit: "px"}
						}});
				} else {
					var anim = dojo.animateProperty({node: this.zoomDiv, duration: 250, 
					properties: 
						{width: {end: this.initialDimensions.w, unit: "px"}, 
						height: {end: this.initialDimensions.h, unit: "px"},
						top: {end: this.initialDimensions.t, unit: "px"},
						left: {end: this.initialDimensions.l, unit: "px"}
						}});	
				}
				//animate height
				var anim2 = dojo.animateProperty({node: this._contentWrapper, duration: 250, properties: {height: {end: this.initialDimensions.h - 24, unit: "px"}}});
	
				anim.play();
				anim2.play();				
			} else {
				this.zoomDiv.style.height = this.initialDimensions.h + "px";
				this.zoomDiv.style.top = this.initialDimensions.t + "px";
				if (this._columns.length != 1) {
					this.zoomDiv.style.width = this.initialDimensions.w + "px";
					this.zoomDiv.style.left = this.initialDimensions.l + "px";
				}
				this._contentWrapper.style.height = this.initialDimensions.h - 24 + "px";
			}

			//make sure zoom icon isn't adjusted for scroll position
			this._zoom.style.bottom = "";
			
			//kill events
			this.disconnect(this._unZoomEvent);
			this.disconnect(this._onViewletClickEvent);
			this.disconnect(this.onViewletScroll);
			ResizeNotifier.unregister(this._zoomWindowResize);
			this._unZoomEvent = null;
			this._zoomWindowResize = null;
			this.onViewletScroll = null;
			
			this.dashboardContainer._hideBlockerDiv();
			
			this.isZoomed = false;
			this.disconnect(this._closeOnEsc);
			
			if (!noAnimation) {
				this.connect(anim, "onEnd", this._onCollapseEnd);
				this.connect(anim2, "onEnd", function(){
					//make sure these get called always
					this._contentWrapper.style.overflow = "hidden"; // Must define this programatically to fix scrollbar bug in IE
					this._contentWrapper.style.height = "";
					this.domNode.style.height = "";
				});	
			} else {
				this._onCollapseEnd();
			}
			this._header.focus();
		}
		//tell viewlet that it should/shouldn't show zoomed content
	},
	_onExpandEnd: function(){
		this._zooming = false;
		// Set zoom state and update viewlet content
		this._viewlet.setZoomed(true);
		// Notify viewlet that a zoom operation has completed
		this._viewlet.postZoom();	
		this._viewlet.getSite().setLoading(false);
		this._unZoomEvent = this.connect(dojo.byId("net-jazz-ajax-WorkbenchRoot"),"onclick",this.toggleZoom);
		dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		this._updateScrollPosition();
		if (this.trim == TrimStyles.TRIMLESS)
			setTimeout(dojo.hitch(this,this._addHoverClass, null, true),150);
	},
	_onCollapseEnd: function(){
		this._zooming = false;
		// Set zoom state and update viewlet content
		this._viewlet.setZoomed(false);	
		// Notify viewlet that a zoom operation has completed
		this._viewlet.postZoom();	
		this._viewlet.getSite().setLoading(false);
		
		this._contentWrapper.style.overflow = "hidden"; // Must define this programatically to fix scrollbar bug in IE
		if (dojo.isIE) {
			this.zoomPlaceholder.parentNode.insertBefore(this.zoomDiv,this.zoomPlaceholder);
			if (this._columnIndex === 0) {
				dojo.removeClass(this.domNode, "column1");	
			}
		}
		this.zoomPlaceholder.parentNode.removeChild(this.zoomPlaceholder);
		var zdStyle = this.zoomDiv.style;
		zdStyle.position = "";
		zdStyle.top = "";
		zdStyle.width = "";
		zdStyle.left = "";
		zdStyle.zIndex = "";
		
		dojo.removeClass(this.zoomDiv, "com-ibm-team-dashboard-zoom-container");
		this.zoomPlaceholder = null;
		this._contentWrapper.style.height = "";
		this.domNode.style.height = "";
	
		this.dashboardContainer.zoomDiv = null;
		this.dashboardContainer._updateZoomMargins = null;
		this.zoomDiv = null;
	
		dojo.removeClass(this._zoom, "zoom-out");
		dojo.addClass(this._zoom, "zoom-in");
		this._zoomImg.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/zoom-in.gif");
	
		dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		this._removeHoverClass(null, true); //show hover trim while zoomed
		if (this.trim == TrimStyles.TRIMLESS)
			this._addHoverClass(null, true);
		this._header.focus();
	},
	addMenuItem: function(/* object */ args) {
		if (!this._contributedMenuItems) {
			this._contributedMenuItems = new Array();
		}
		this._contributedMenuItems.push(args);
	},
	_createMenu: function (){
		if (this._menu) {
			this._menu.destroy();
		}
		this.menuItems = new Array();
		
		// index to add viewlet menu contributions
		this._menuContribIndex = 0;
		
		if (this._state == STATE_ACTIVE) {
			if (this.editing) {
				this.menuItems.push({label: messages.editSettings, iconClass: 'prefsIcon', onClick: dojo.hitch(this,this.toggleSettings)});
				this._menuContribIndex++
				if (this._viewletDefinition && this._viewletDefinition.editable) {
					this.menuItems.push({label: messages.editContents, iconClass: 'editIcon', onClick: dojo.hitch(this,this.startEditing)});
					this._menuContribIndex++
				}
				this.menuItems.push({separator: true, label: "Separator"});
				this._menuContribIndex++
			}
			
			// add known menu contributions
			if (this._contributedMenuItems) {
				for (var i=0;i<this._contributedMenuItems.length;i++) {
					this.menuItems.push(this._contributedMenuItems[i]);
				}
				this._menuContribIndex += this._contributedMenuItems.length;
				this.menuItems.push({separator: true, label: "Separator"});
			}
		}
		
		if (this._state == STATE_ACTIVE || this._state == STATE_LOGIN) {
			// read-only viewlets are still refreshable
			this.menuItems.push({label: messages.refresh, iconClass: 'refreshIcon', onClick: dojo.hitch(this,this.refreshViewlet)});
		}
		
		if (this._state == STATE_ACTIVE) {
			if (this.editing) {
				this.menuItems.push({separator: true, label: "Separator"});
				this.menuItems.push({
					label: messages.copyViewlet,
					iconClass: 'copyIcon',
					onClick: dojo.hitch(this, this.copyViewlet)
				});
				this.menuItems.push({
					label: messages.duplicateViewlet,
					iconClass: 'cloneIcon',
					onClick: dojo.hitch(this, this.cloneViewlet)
				});
			}
		}
		
		this.inherited(arguments);
		dojo.addClass(this._menu.domNode, "com-ibm-team-dashboard-content-menu");
	},
	_openMenu: function(event) {
		if (this.viewletPage.killPopup){
			this.viewletPage.killPopup();
		}
		this.inherited(arguments);
		this.viewletPage.killPopup = dojo.hitch(this,this._closeMenu);
	},
	_closeMenu: function(event) {
		this.viewletPage.killPopup = null;
		this.inherited(arguments);
	},
	_hideZoom: function(e) {
		dojo.removeClass(this._zoom, "zoom-visible");
	},
	_showZoom: function(e) {
		if (this._viewlet && this._viewlet.hasZoomableContent())
			dojo.addClass(this._zoom, "zoom-visible");
		else return;
		
		if (this.isZoomed) {
			if (!(dojo.isIE < 7 && dojo.isIE > 0)) {
				this._zoom.style.bottom = 1 - this._contentWrapper.scrollTop + "px";	
			}
			if (!this.onViewletScroll) {
				this.onViewletScroll = this.connect(this._contentWrapper,"onscroll",this._showZoom);	
			}
		}
	},	
	_calcZoomVariables: function() {
		var marginLeft = dojo.style(this.domNode,"marginLeft");
		var marginTop = dojo.style(this.domNode,"marginTop");
		var marginRight = dojo.style(this.domNode,"marginRight");
		
		var dc = this.dashboardContainer;
		//get coordinates for various elements of the page
		if (this.zoomPlaceholder){
			var coords = dojo.coords(this.zoomPlaceholder, true);
			var viewletPageCoords = dojo.coords(this.zoomPlaceholder.parentNode.parentNode, true);
		} else {
			var coords = dojo.coords(this.domNode, true);
			var viewletPageCoords = dojo.coords(this.domNode.parentNode.parentNode, true);
		}
		var dashboardCoords = dojo.isIE ? dojo.coords(document.body, true) : dojo.coords(dc.domNode,true);
		var workbenchCoords = dojo.byId("content-cell") ? dojo.coords(dojo.byId("content-cell"),true) : dashboardCoords;
		
		//calculate bounds
		var minTop = viewletPageCoords.y - dashboardCoords.y;
		var minLeft = viewletPageCoords.x - dashboardCoords.x;

		//calculate initial dimensions
		if (this._columns.length == 1) {
			var initWidth = coords.w;
		} else {
			var initWidth = coords.w - marginRight - marginLeft;
		}
		if (this.zoomPlaceholder) {
			this.initialDimensions = {
				l: coords.x - dashboardCoords.x - marginLeft,
				t: coords.y - dashboardCoords.y,
				w: initWidth,
				h: coords.h - marginTop
			}
		} else {
			this.initialDimensions = {
				l: coords.x - dashboardCoords.x - marginLeft,
				t: coords.y - dashboardCoords.y - marginTop,
				w: initWidth,
				h: coords.h - marginTop
			}
		} 
		//calculate zoom height/width
		try {
			var viewportWidth = document.body.clientWidth;
			var viewportHeight = window.innerHeight;
			if (!viewportHeight) viewportHeight = document.documentElement.clientHeight;	
		} catch(e){
			viewportWidth = 640;
			viewportHeight = 480;
		}
		if (this._columns.length != 1) {
			var endWidth = Math.round(viewportWidth * 0.60);
		} else {
			var endWidth = initWidth;
		}
		var endHeight = Math.min(Math.round(viewportHeight * 0.70), workbenchCoords.h - dc.metaNav.domNode.parentNode.clientHeight - 20);
		
		//calculate zoom positions
		var endTop = this.initialDimensions.t - (endHeight - this.initialDimensions.h) / 2;
		var endLeft = this.initialDimensions.l - (endWidth - this.initialDimensions.w) / 2;
			
		//if zoom will cause the viewlet to expand outside bounds of page, constrain it to bounds
		if (endLeft < minLeft) endLeft = minLeft;
		if (endLeft + endWidth > minLeft + viewletPageCoords.w) endLeft = minLeft + viewletPageCoords.w - endWidth - marginRight - marginLeft;
		if (endTop < minTop) endTop = minTop;
		if (endTop + endHeight + marginTop > workbenchCoords.h - 20) endTop = workbenchCoords.h - 20 - endHeight;
		
		this.endDimensions = {
			l: endLeft,
			t: endTop,
			w: endWidth,
			h: endHeight
		}
	},
	_updateZoomMargins: function() {
		if (this._zooming) {
			setTimeout(dojo.hitch(this, this._updateZoomMargins), 200);
			return;
		}
		if (!this.zoomDiv) return;
		
		this._calcZoomVariables();
		
		this.zoomDiv.style.top = this.endDimensions.t + "px";
		this.zoomDiv.style.height = this.endDimensions.h + "px";
		this._contentWrapper.style.height = this.endDimensions.h + "px";
		if (this._columns.length > 1) {
			this.zoomDiv.style.left = this.endDimensions.l + "px";
			this.zoomDiv.style.width = this.endDimensions.w + "px";
		}
		this._updateScrollPosition();
	},
	_updateScrollPosition: function() {
		var coords = dojo.coords(this.zoomDiv);
		var dy = coords.y + this.endDimensions.h - document.documentElement.clientHeight + 30;
		if (dy > 0) {
			window.scrollBy(0,dy);
		} else if (coords.y < 0) {
			window.scrollBy(0,coords.y-20);
		}
	},
	destroy: function() {
		// kill auto-refresh
		if (this._refreshTimeout) {
			clearTimeout(this._refreshTimeout);
			delete this._refreshTimeout;
		}
		// kill any remaining timeouts
		if (this._yieldTimeout) {
			// cancel yield
			clearTimeout(this._yieldTimeout);
			delete this._yieldTimeout;
		}
		// abort any remaining requests
		if (this._serviceTracker) {
			var requests = this._serviceTracker.getActiveRequests();
			for (var i=0;i<requests.length;i++) {
				requests[i].abort();
			}
		}
		// stop tracking log-ins for remote viewlets
		this._removeLoginTracker();
		// destroy the widgets
		if (this._settingsWidget && this._settingsWidget.destroy) {
			this._settingsWidget.destroy();
		}
		if (this._viewlet) {
			try {
				this._viewlet.destroy();
			} catch (e) {
				// iframe may be in the process of unloading; ignore
			}
			delete this._viewlet;
		}
		if (this._iframeContainer) {
			this._iframeContainer.destroy();
			delete this._iframeContainer;
		}
		if (this._errorReport) {
			this._errorReport.destroy();
			delete this._errorReport;
		}
		if (this.zoomDiv && this.zoomDiv.parentNode) {
			this.zoomDiv.parentNode.removeChild(this.zoomDiv);
		}
		if (this._zoomWindowResize) {
			ResizeNotifier.unregister(this._zoomWindowResize);
			this._zoomWindowResize = null;	
		}
		this._edit = null;
		this._close = null;
		this._isDestroyed = true;
		this.inherited(arguments);
	},
	/*
	 * Utility empty callback for success responses we don't need to
	 * react to.
	 */
	_empty: function() {
	},
	/*
	 * Utility method to log remote errors when calling services.
	 */	
	_logError: function(errorObj) {
		console.error("ViewletContainer.js remote error: " + errorObj.message);
	},
	_onChange: function(/* string */ property) {
		// notify listeners that the content has changed
		this.propertyChanged(property);
	},
	/* ServiceTracker */ _getServiceTracker: function() {
		if (!this._serviceTracker) {
			this._serviceTracker = new ServiceTracker();
		}
		return this._serviceTracker;
	}
});

})();
