/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.AccessControlEditor"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dijit.Tree"); //$NON-NLS-1$
	dojo.require("dojo.data.ItemFileWriteStore"); //$NON-NLS-1$
	dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.client.init.InitializationData"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.projectarea"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.TRToolbar"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	
	var USER_EDITOR = "com.ibm.team.repository.editUser"; //$NON-NLS-1$
	var ARCHIVED_USER_EDITOR = "com.ibm.team.repository.editArchivedUser"; //$NON-NLS-1$
	var PAGE_SIZE = 25;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.AccessControlEditor", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/AccessControlEditor.html"), //$NON-NLS-1$ //$NON-NLS-2$
		constructor: function(args){
			this.editor = args.editor;
			this.accessControl = args.editor.accessControl;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		
		postCreate: function(){
		
			this._connectHandles = [];
			this._actionToolbars = [];
			this.connect(this.publicVisibility, "onclick", "handleChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.memberVisibility, "onclick", "handleChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.adminVisibility, "onclick", "handleChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.memberAndUserVisibility, "onclick", "handleChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.specificUsersVisibility, "onclick", "handleChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.memberAddButton, "onclick", function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				this._addUserDialog(this._getPendingMembersToFilter(), "_addMembersToAdd"); //$NON-NLS-1$
			});
			this.connect(this.memberRemoveButton, "onclick", function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				this._addUserDialog(this._getPendingMembersToFilter(), "_addMembersToRemove"); //$NON-NLS-1$
			});
			
			this.connect(this.searchText, "onkeypress", function(e){ //$NON-NLS-1$
				if (e.keyCode == 13) {
					this._searchForMembersWithReadAccess();
				}
			});
			this.connect(this.searchButton, "onclick", function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				this._searchForMembersWithReadAccess();
			});
			this.refresh();
			
		},
		
		refresh: function(){
			this._destroyActionToolbars();
			
			this.accessControl = this.editor.accessControl;
			this.clearState();
			this.editor.addSaveParticipant(this, "com.ibm.team.process.web.ui.internal.admin.projects.AccessControlEditor"); //$NON-NLS-1$
			
			dojo.toggleClass(this.pendingMembersArea, "hidden", true); //$NON-NLS-1$
			if (this.accessControl.publicVisible) {
				this._initializeVisibility(this.publicVisibility);
			} else if (this.accessControl.memberVisible && this.accessControl.specificUserVisible) {
				this._initializeVisibility(this.memberAndUserVisibility);
				dojo.toggleClass(this.pendingMembersArea, "hidden", false); //$NON-NLS-1$
			} else if (this.accessControl.memberVisible) {
				this._initializeVisibility(this.memberVisibility);
			} else if (this.accessControl.specificUserVisible) {
				this._initializeVisibility(this.specificUsersVisibility);
				dojo.toggleClass(this.pendingMembersArea, "hidden", false); //$NON-NLS-1$
			} else {
				this._initializeVisibility(this.adminVisibility);
			}
			
			this.populateUsers(this.pendingMemberChanges.add, this.memberAddHeader, this.memberAddArea, this._addMemberRemoveHandler);
			this.populateUsers(this.pendingMemberChanges.remove, this.memberRemoveHeader, this.memberRemoveArea, this._removeMemberRemoveHandler);
			if (this.accessControl.specificUserVisible) {
				var srh = new ServiceResponseHandler(this, "populateMembersWithReadAccess", "handleFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getMembersWithReadAccess({
					projectAreaItemId: this.editor.projectAreaItemId,
					pageNum: this.pageNum,
					pageSize: PAGE_SIZE
				}, srh);
			}
			
		},
		
		_initializeVisibility: function(radioButton){
			radioButton.checked = true;
			// Workitem 80966...IE 7 needs the defaultChecked state set too
			// or it will not properly set the radio button upon tab creation
			radioButton.defaultChecked = true;
			radioButton.savedValue = "on"; //$NON-NLS-1$
		},
		
		_searchForMembersWithReadAccess: function(){
			var searchTerm = ""; //$NON-NLS-1$
			if (dojo.string.trim(this.searchText.value).length > 0) {
				searchTerm = this.searchText.value.replace(/\*/g, "%"); //$NON-NLS-1$
			}
			
			var srh = new ServiceResponseHandler(this, "populateMembersWithReadAccessFromSearch", "handleFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getMembersWithReadAccess({
				projectAreaItemId: this.editor.projectAreaItemId,
				pageNum: this.pageNum,
				pageSize: PAGE_SIZE,
				searchTerm: searchTerm
			}, srh);
			
		},
		
		_addUserDialog: function(filterUsers, callbackFunction){
		
			var userPicker = new com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog({
				'disableUsers': filterUsers //$NON-NLS-1$
			});
			this.connect(userPicker, "onAdd", callbackFunction); //$NON-NLS-1$
		},
		
		_getPendingMembersToFilter: function(){
			var pendingMembersToFilter = [];
			
			// Add any pending members to add
			for (var i = 0; i < this.pendingMemberChanges.add.length; i++) {
				pendingMembersToFilter.push({
					'itemId': this.pendingMemberChanges.add[i].itemId //$NON-NLS-1$
				});
			}
			
			// Also filter out any pending members to remove
			for (i = 0; i < this.pendingMemberChanges.remove.length; i++) {
				pendingMembersToFilter.push({
					'itemId': this.pendingMemberChanges.remove[i].itemId //$NON-NLS-1$
				});
			}
			
			return pendingMembersToFilter;
			
		},
		
		_addMembersToAdd: function(newMembers){
			if (newMembers != null) {
				if (newMembers.length > 0) {
					this.editor.setDirty(true);
					dojo.forEach(newMembers, function(member){
						this.pendingMemberChanges.add.push(member);
					}, this);
					this.populateUsers(this.pendingMemberChanges.add, this.memberAddHeader, this.memberAddArea, this._addMemberRemoveHandler);
				}
			}
		},
		
		_addMembersToRemove: function(newMembers){
			if (newMembers != null) {
				if (newMembers.length > 0) {
					this.editor.setDirty(true);
					dojo.forEach(newMembers, function(member){
						this.pendingMemberChanges.remove.push(member);
					}, this);
					this.populateUsers(this.pendingMemberChanges.remove, this.memberRemoveHeader, this.memberRemoveArea, this._removeMemberRemoveHandler);
				}
			}
		},
		
		_addMemberRemoveHandler: function(e){
			this._removeAPendingMember(e, this.pendingMemberChanges.add);
		},
		
		_removeMemberRemoveHandler: function(e){
			this._removeAPendingMember(e, this.pendingMemberChanges.remove);
		},
		
		_removeAPendingMember: function(event, pendingMembers){
			dojo.stopEvent(event);
			var rowNode = event.target;
			while (rowNode.tagName !== "TR") { //$NON-NLS-1$
				rowNode = rowNode.parentNode;
			}
			var selectedUser = rowNode.userObj;
			var itemId = selectedUser.itemId;
			
			for (var j = 0; j < pendingMembers.length; j++) {
				if (itemId === pendingMembers[j].itemId) {
					pendingMembers.splice(j, 1);
					break;
				}
			}
			this.editor.setDirty(this.editor.changesPending());
			rowNode.parentNode.removeChild(rowNode);
			
		},
		
		_removeMemberWithAccess: function(event){
			dojo.stopEvent(event);
			var rowNode = event.target;
			while (rowNode.tagName !== "TR") { //$NON-NLS-1$
				rowNode = rowNode.parentNode;
			}
			var selectedUser = rowNode.userObj;
			var itemId = selectedUser.itemId;
			var found = false;
			
			for (var j = 0; j < this.pendingMemberChanges.remove.length; j++) {
				if (itemId === this.pendingMemberChanges.remove[j].itemId) {
					found = true;
					break;
				}
			}
			if (!found) {
				this.pendingMemberChanges.remove.push(selectedUser);
				this.populateUsers(this.pendingMemberChanges.remove, this.memberRemoveHeader, this.memberRemoveArea, this._removeMemberRemoveHandler);
				// Don't leave the focus lines when the HTML moves down
				// from adding a line to the pending remove list
				event.currentTarget.blur();
			}
			this.editor.setDirty(this.editor.changesPending());
			
		},
		
		populateMembersWithReadAccessFromSearch: function(resultSet){
			this.populateMembersWithReadAccess(resultSet, true);
		},
		
		populateMembersWithReadAccess: function(resultSet, fromSearch){
			if (resultSet && resultSet.elements && resultSet.elements.length > 0) {
				if (this.pagerDiv == null) {
					this.createPagerDiv();
				}
				dojox.data.dom.removeChildren(this.pagerDiv.rangeSpan);
				dojox.data.dom.removeChildren(this.pagerDiv.totalSpan);
				if (resultSet.count > PAGE_SIZE) {
					var lowerBound = (this.pageNum * PAGE_SIZE) + 1;
					var upperBound = (this.pageNum + 1) * PAGE_SIZE;
					if (this.pageNum == 0) {
						dojo.toggleClass(this.pagerDiv.prevSpan, "disabled", true); //$NON-NLS-1$
					} else {
						dojo.toggleClass(this.pagerDiv.prevSpan, "disabled", false); //$NON-NLS-1$
					}
					if (upperBound >= resultSet.count) {
						upperBound = resultSet.count;
						dojo.toggleClass(this.pagerDiv.nextSpan, "disabled", true); //$NON-NLS-1$
					} else {
						dojo.toggleClass(this.pagerDiv.nextSpan, "disabled", false); //$NON-NLS-1$
					}
					this.pagerDiv.rangeSpan.appendChild(document.createTextNode(lowerBound + " - " + upperBound)); //$NON-NLS-1$
				} else {
					dojo.toggleClass(this.pagerDiv.prevSpan, "disabled", true); //$NON-NLS-1$
					dojo.toggleClass(this.pagerDiv.nextSpan, "disabled", true); //$NON-NLS-1$
					this.pagerDiv.rangeSpan.appendChild(document.createTextNode("1 - " + resultSet.count)); //$NON-NLS-1$
				}
				this.pagerDiv.totalSpan.appendChild(document.createTextNode(resultSet.count));
				this.populateUsers(resultSet.elements, this.memberHeader, this.memberArea, "_removeMemberWithAccess"); //$NON-NLS-1$
			} else {
				if (this.pagerDiv) {
					dojox.data.dom.removeChildren(this.pagerDiv);
					this.pagerDiv = null;
				}
				
				//Create the table header
				this.populateUsers(resultSet.elements, this.memberHeader, this.memberArea, "_removeMemberWithAccess"); //$NON-NLS-1$    
				// Create a no users with granted access row            
				var tr = document.createElement("tr"); //$NON-NLS-1$
				var td = document.createElement("td"); //$NON-NLS-1$
				td.className = "aclMemberName"; //$NON-NLS-1$
				if (fromSearch) {
					td.appendChild(document.createTextNode(this.messages.AccessControlEditor_NoUsersGrantedAccessThatMatchSearch));
				} else {
					td.appendChild(document.createTextNode(this.messages.AccessControlEditor_NoUsersGrantedAccess));
				}
				td.colSpan = 4;
				tr.appendChild(td);
				this.memberArea.appendChild(tr);
				
			}
			
			
		},
		
		handleFetchError: function(error){
			this.editor.handleServerResponseError(error, this.messages.AccessControlEditor_ErrorFetchingMembers);
		},
		
		populateUsers: function(users, header, userArea, removeHandler){
			var self = this;
			dojox.data.dom.removeChildren(header);
			var tr = document.createElement("tr"); //$NON-NLS-1$
			tr.className = "headerLineRow"; //$NON-NLS-1$
			var th = document.createElement("th"); //$NON-NLS-1$
			if (dojo.isIE) {
				th.appendChild(document.createTextNode(" ")); //$NON-NLS-1$
			}
			th.className = "aclMemberName"; //$NON-NLS-1$
			tr.appendChild(th);
			
			th = document.createElement("th"); //$NON-NLS-1$
			th.className = "aclMemberId"; //$NON-NLS-1$
			if (dojo.isIE) {
				th.appendChild(document.createTextNode(" ")); //$NON-NLS-1$
			}
			tr.appendChild(th);
			
			th = document.createElement("th"); //$NON-NLS-1$
			th.className = "aclMemberMail"; //$NON-NLS-1$
			if (dojo.isIE) {
				th.appendChild(document.createTextNode(" ")); //$NON-NLS-1$
			}
			tr.appendChild(th);
			
			th = document.createElement("th"); //$NON-NLS-1$
			th.className = "iconNode"; //$NON-NLS-1$
			if (dojo.isIE) {
				th.appendChild(document.createTextNode(" ")); //$NON-NLS-1$
			}
			tr.appendChild(th);
			
			header.appendChild(tr);
			tr = document.createElement("tr"); //$NON-NLS-1$
			th = document.createElement("th"); //$NON-NLS-1$
			th.appendChild(document.createTextNode(this.messages.AccessControlEditor_UserTableNameHeader));
			th.className = "aclMemberName"; //$NON-NLS-1$
			tr.appendChild(th);
			
			th = document.createElement("th"); //$NON-NLS-1$
			th.appendChild(document.createTextNode(this.messages.AccessControlEditor_UserTableUserIdHeader));
			th.className = "aclMemberId"; //$NON-NLS-1$
			tr.appendChild(th);
			
			th = document.createElement("th"); //$NON-NLS-1$
			th.appendChild(document.createTextNode(this.messages.AccessControlEditor_UserTableEmailHeader));
			th.className = "aclMemberMail"; //$NON-NLS-1$
			tr.appendChild(th);
			th = document.createElement("th"); //$NON-NLS-1$
			th.className = ProcessConstants.CSSClass.ACTION_COLUMN;
			th.appendChild(document.createTextNode(this.messages.AccessControlEditor_UserTableActionsHeader));
			tr.appendChild(th);
			header.appendChild(tr);
			dojox.data.dom.removeChildren(userArea);
			if (users != null && users.length > 0) {
				for (var i = 0; i < users.length; i++) {
					tr = document.createElement("tr"); //$NON-NLS-1$
					tr.userObj = users[i];
					
					// Member name
					var td = document.createElement("td"); //$NON-NLS-1$
					td.className = "aclMemberName"; //$NON-NLS-1$
					var link = document.createElement("a"); //$NON-NLS-1$
					if (users[i].archived) {
						link.href = "#action=" + ARCHIVED_USER_EDITOR + "&itemId=" + users[i].itemId; //$NON-NLS-1$ //$NON-NLS-2$
					} else {
						link.href = "#action=" + USER_EDITOR + "&itemId=" + users[i].itemId; //$NON-NLS-1$ //$NON-NLS-2$
					}
					link.appendChild(document.createTextNode(users[i].name));
					this._connectHandles.push(dojo.connect(link, "onfocus", tr, function(e){ //$NON-NLS-1$
						dojo.toggleClass(this, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, true);
					}));
					this._connectHandles.push(dojo.connect(link, "onblur", tr, function(e){ //$NON-NLS-1$
						dojo.toggleClass(this, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, false);
					}));
					td.appendChild(link);
					tr.appendChild(td);
					
					// Member id
					td = document.createElement("td"); //$NON-NLS-1$
					td.className = "aclMemberId"; //$NON-NLS-1$
					link = document.createElement("a"); //$NON-NLS-1$
					if (users[i].archived) {
						link.href = "#action=" + ARCHIVED_USER_EDITOR + "&itemId=" + users[i].itemId; //$NON-NLS-1$ //$NON-NLS-2$
					} else {
						link.href = "#action=" + USER_EDITOR + "&itemId=" + users[i].itemId; //$NON-NLS-1$ //$NON-NLS-2$
					}
					link.appendChild(document.createTextNode(users[i].userId));
					this._connectHandles.push(dojo.connect(link, "onfocus", tr, function(e){ //$NON-NLS-1$
						dojo.toggleClass(this, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, true);
					}));
					this._connectHandles.push(dojo.connect(link, "onblur", tr, function(e){ //$NON-NLS-1$
						dojo.toggleClass(this, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, false);
					}));
					
					td.appendChild(link);
					tr.appendChild(td);
					
					// Member email
					td = document.createElement("td"); //$NON-NLS-1$
					td.className = "aclMemberMail"; //$NON-NLS-1$
					if (users[i].emailAddress) {
						link = document.createElement("a"); //$NON-NLS-1$
						link.href = "mailto:" + users[i].emailAddress; //$NON-NLS-1$
						link.appendChild(document.createTextNode(users[i].emailAddress));
						td.appendChild(link);
						this._connectHandles.push(dojo.connect(link, "onfocus", tr, function(e){ //$NON-NLS-1$
							dojo.toggleClass(this, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, true);
						}));
						this._connectHandles.push(dojo.connect(link, "onblur", tr, function(e){ //$NON-NLS-1$
							dojo.toggleClass(this, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, false);
						}));
					} else if (dojo.isIE) {
						td.appendChild(document.createTextNode(" ")); //$NON-NLS-1$
					}
					tr.mailDisplay = td;
					tr.appendChild(td);
					
					td = document.createElement("td"); //$NON-NLS-1$
					var trToolbar = new com.ibm.team.process.web.ui.internal.TRToolbar();
					var removeAction = {};
					removeAction.imgSrc = dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/remove.gif"); //$NON-NLS-1$  //$NON-NLS-2$
					removeAction.label = this.messages.AccessControlEditor_RemoveIconLabel;
					trToolbar.addButton(this, removeHandler, removeAction);
					td.appendChild(trToolbar.domNode);
					this._actionToolbars.push(trToolbar);
					
					tr.appendChild(td);
					this._connectHandles.push(dojo.connect(tr, "onmouseover", tr, function(e){ //$NON-NLS-1$
						dojo.toggleClass(this, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, true);
					}));
					this._connectHandles.push(dojo.connect(tr, "onmouseout", function(e){ //$NON-NLS-1$
						dojo.toggleClass(this, ProcessConstants.CSSClass.TABLE_ROW_SELECTED, false);
					}));
					userArea.appendChild(tr);
				}
			}
		},
		
		createPagerDiv: function(){
			this.pagerDiv = document.createElement("div"); //$NON-NLS-1$
			dojo.toggleClass(this.pagerDiv, "pager", true); //$NON-NLS-1$
			var prevSpan = document.createElement("span"); //$NON-NLS-1$
			var prevLink = document.createElement("a"); //$NON-NLS-1$
			prevLink.href = "#"; //$NON-NLS-1$
			prevLink.appendChild(document.createTextNode(this.messages.AccessControlEditor_TableControlsPrevious));
			prevSpan.appendChild(prevLink);
			var disabledPrev = document.createElement("span"); //$NON-NLS-1$
			disabledPrev.appendChild(document.createTextNode(this.messages.AccessControlEditor_TableControlsPrevious));
			dojo.toggleClass(disabledPrev, "disabledText", true); //$NON-NLS-1$
			prevSpan.appendChild(disabledPrev);
			this.pagerDiv.appendChild(prevSpan);
			this.pagerDiv.prevSpan = prevSpan;
			this.pagerDiv.prevLink = prevLink;
			this.pagerDiv.appendChild(document.createTextNode(" | ")); //$NON-NLS-1$
			var rangeSpan = document.createElement("span"); //$NON-NLS-1$
			dojo.toggleClass(rangeSpan, "bold", true); //$NON-NLS-1$
			this.pagerDiv.appendChild(rangeSpan);
			this.pagerDiv.rangeSpan = rangeSpan;
			this.pagerDiv.appendChild(document.createTextNode(" " + this.messages.AccessControlEditor_TableControlsOf + " ")); //$NON-NLS-1$ //$NON-NLS-2$
			var totalSpan = document.createElement("span"); //$NON-NLS-1$
			dojo.toggleClass(totalSpan, "bold", true); //$NON-NLS-1$
			this.pagerDiv.appendChild(totalSpan);
			this.pagerDiv.totalSpan = totalSpan;
			this.pagerDiv.appendChild(document.createTextNode(" | ")); //$NON-NLS-1$
			var nextSpan = document.createElement("span"); //$NON-NLS-1$
			var nextLink = document.createElement("a"); //$NON-NLS-1$
			nextLink.href = "#"; //$NON-NLS-1$
			nextLink.appendChild(document.createTextNode(this.messages.AccessControlEditor_TableControlsNext)); //$NON-NLS-1$
			nextSpan.appendChild(nextLink);
			var disabledNext = document.createElement("span"); //$NON-NLS-1$
			disabledNext.appendChild(document.createTextNode(this.messages.AccessControlEditor_TableControlsNext));
			dojo.toggleClass(disabledNext, "disabledText", true); //$NON-NLS-1$
			nextSpan.appendChild(disabledNext);
			this.pagerDiv.appendChild(nextSpan);
			this.pagerDiv.nextSpan = nextSpan;
			this.pagerDiv.nextLink = nextLink;
			this.tableControls.appendChild(this.pagerDiv);
			var self = this;
			this.connect(prevLink, "onclick", function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				self.pageNum--;
				var srh = new ServiceResponseHandler(this, "populateMembersWithReadAccess", "handleFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getMembersWithReadAccess({
					projectAreaItemId: this.editor.projectAreaItemId,
					pageNum: this.pageNum,
					pageSize: PAGE_SIZE
				}, srh);
			});
			this.connect(nextLink, "onclick", function(e){ //$NON-NLS-1$
				dojo.stopEvent(e);
				self.pageNum++;
				var srh = new ServiceResponseHandler(this, "populateMembersWithReadAccess", "handleFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
				ProcessClient.getMembersWithReadAccess({
					projectAreaItemId: this.editor.projectAreaItemId,
					pageNum: this.pageNum,
					pageSize: PAGE_SIZE
				}, srh);
			});
		},
		
		changesPending: function(){
			if (this.publicVisibility.changePending) {
				return true;
			}
			if (this.memberVisibility.changePending) {
				return true;
			}
			
			if (this.adminVisibility.changePending) {
				return true;
			}
			
			if (this.memberAndUserVisibility.changePending) {
				return true;
			}
			if (this.specificUsersVisibility.changePending) {
				return true;
			}
			
			if (this.specificUsersVisibility.checked && this.pendingMemberChanges.add.length > 0 || this.pendingMemberChanges.remove.length > 0) {
				return true;
			}
			
			return false;
		},
		
		handleChange: function(e){
			var buttons = [this.memberVisibility, this.publicVisibility, this.adminVisibility, this.memberAndUserVisibility, this.specificUsersVisibility];
			for (var i = 0; i < buttons.length; i++) {
				buttons[i].changePending = false;
				buttons[i].defaultChecked = false;
			}
			// Workitem 80966...IE 7 needs the defaultChecked state set in the line
			// above and below or it will not properly set the radio button
			// when switching tabs
			e.target.defaultChecked = e.target.checked;
			
			if (e.target === this.memberAndUserVisibility || e.target === this.specificUsersVisibility) {
				if (e.target.checked) {
					dojo.toggleClass(this.pendingMembersArea, "hidden", false); //$NON-NLS-1$
					var srh = new ServiceResponseHandler(this, "populateMembersWithReadAccess", "handleFetchError"); //$NON-NLS-1$ //$NON-NLS-2$
					ProcessClient.getMembersWithReadAccess({
						projectAreaItemId: this.editor.projectAreaItemId,
						pageNum: this.pageNum,
						pageSize: PAGE_SIZE
					}, srh);
				}
			} else {
				if (e.target.checked) {
					dojo.toggleClass(this.pendingMembersArea, "hidden", true); //$NON-NLS-1$
				}
			}
			
			if (e.target.value != e.target.savedValue) {
				e.target.changePending = true;
				this.editor.setDirty(true);
			} else {
				e.target.changePending = false;
				if (!this.editor.changesPending()) {
					this.editor.setDirty(false);
				}
			}
			
		},
		
		aboutToBeSaved: function(saveObj){
			saveObj.publicVisible = false;
			saveObj.memberVisible = false;
			saveObj.specificUserVisible = false;
			
			if (this.publicVisibility.checked) {
				saveObj.publicVisible = true;
			}
			if (this.memberVisibility.checked || this.memberAndUserVisibility.checked) {
				saveObj.memberVisible = true;
			}
			if (this.memberAndUserVisibility.checked || this.specificUsersVisibility.checked) {
				saveObj.specificUserVisible = true;
				saveObj.membersToGiveReadAccess = this._getItemIdsForPendingMembers(this.pendingMemberChanges.add);
				saveObj.membersToRemoveReadAccess = this._getItemIdsForPendingMembers(this.pendingMemberChanges.remove);
			}
		},
		
		_getItemIdsForPendingMembers: function(pendingMembers){
			var pendingMemberIds = [];
			
			for (var i = 0; i < pendingMembers.length; i++) {
				pendingMemberIds.push(pendingMembers[i].itemId);
			}
			
			return pendingMemberIds;
			
		},
		
		saved: function(){
			// See note in ProjectAreaEditor.populateProcessAreaData
			// this.refresh();
		},
		
		clearState: function(){
			this.editor.removeSaveParticipant("com.ibm.team.process.web.ui.internal.admin.projects.AccessControlEditor"); //$NON-NLS-1$
			this.pendingMemberChanges = {};
			this.pendingMemberChanges.add = [];
			this.pendingMemberChanges.remove = [];
			
			this._resetAccessControlRadioButton(this.publicVisibility);
			this._resetAccessControlRadioButton(this.memberVisibility);
			this._resetAccessControlRadioButton(this.adminVisibility);
			this._resetAccessControlRadioButton(this.memberAndUserVisibility);
			this._resetAccessControlRadioButton(this.specificUsersVisibility);
			
			this.pageNum = 0;
			this.searchText.value = ""; //$NON-NLS-1$
		},
		
		_resetAccessControlRadioButton: function(radioButton){
			radioButton.changePending = false;
			radioButton.checked = false;
			radioButton.savedValue = "off"; //$NON-NLS-1$
		},
		
		_disableCheckBox: function(checkbox, span, disable){
			checkbox.disabled = disable;
			dojo.toggleClass(span, "disabled", disable); //$NON-NLS-1$
		},
		
		_destroyActionToolbars: function(){
			for (var i = 0; i < this._actionToolbars.length; i++) {
				this._actionToolbars[i].destroyRecursive();
			}
			this._actionToolbars = [];
		},
		
		uninitialize: function(){
			this.editor.removeSaveParticipant("com.ibm.team.process.web.ui.internal.admin.projects.AccessControlEditor"); //$NON-NLS-1$
			this._destroyActionToolbars();
			dojo.forEach(this._connectHandles, function(handle){
				dojo.disconnect(handle);
			});
			
			delete this.editor;
			delete this.accessControl;
			this.inherited(arguments);
		}
		
	});
	
})();
