/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.TimelineDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.Button"); //$NON-NLS-1$
dojo.require("dijit.form.TextBox"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojox.data.dom"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.DateTimeTextBox"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
(function(){
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.TimelineDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/TimelineDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		widgetsInTemplate: true,
		
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function(){
		
			if (this.isNewTimeline === true) {
				this.projectTimelineCheckBoxLabel.innerHTML = this.messages.TimelineDialog_NewProjectTimelineCheckBoxLabel;
			} else {
				this.projectTimelineCheckBoxLabel.innerHTML = this.messages.TimelineDialog_ExistingProjectTimelineCheckBoxLabel;
			}
			this.inputFields = [this.identifier, this.displayName, this.startDate, this.endDate];
			this.identifier.attr('intermediateChanges', true); //$NON-NLS-1$
			var self = this;
			this.identifier.onChange = function(newValue){
				self._validateData();
			};
			this.startDate.attr('intermediateChanges', true); //$NON-NLS-1$
			this.startDate.onChange = function(newValue){
				self._validateData();
				
			};
			this.endDate.attr('intermediateChanges', true); //$NON-NLS-1$
			this.endDate.onChange = function(newValue){
				self._validateData();
				
			};
			
			this._populateDialog();
		},
		
		startup: function(){
			com.ibm.team.process.web.ui.internal.admin.projects.UIHelper.resizeI18NButtons(this.dialogFooter, [this.finishButton, this.cancelButton]);
			this.inherited(arguments);
		},
		
		_validateData: function(){
			if (this._isValidIdentifier() && this._isValidDateRange()) {
				this.finishButton.disabled = false;
			} else {
				this.finishButton.disabled = true;
			}
			
		},
		
		_isValidDateRange: function(){
			var startDate = this.startDate.attr('value'); //$NON-NLS-1$
			var endDate = this.endDate.attr('value'); //$NON-NLS-1$
			if (startDate !== null && !this.startDate.isValid()) {
				return false; // Start date must contain a valid date
			}
			if (endDate === null) {
				return true; // Always ok if there is no end date
			}
			if (startDate === null && endDate !== null) {
				return false; // Can't have end date without start date
			}
			
			return endDate >= startDate; // End date must be greater than or equal to the start date
		},
		
		_isValidIdentifier: function(){
			var identifier = this.identifier.attr('value'); //$NON-NLS-1$
			return identifier && dojo.string.trim(identifier).length > 0;
		},
		
		_populateDialog: function(){
			this.projectTimelineCheckBox.checked = this.isProjectTimeline;
			for (var i = 0; i < this.inputFields.length; i++) {
				this.inputFields[i].attr('value', this.initialValues[i]); //$NON-NLS-1$
			}
		},
		
		changesPending: function(){
			if (this.isNewTimeline === true) {
				return true;
			}
			
			for (var i = 0; i < this.inputFields.length; i++) {
				if (this._compare(this.inputFields[i].attr("value"), this.initialValues[i]) != 0) { //$NON-NLS-1$
					return true;
				}
			}
			
			if (this.isProjectTimeline !== this.projectTimelineCheckBox.checked) {
				return true;
			}
			return false;
		},
		
		_compare: function(newValue, oldValue){
			if (newValue > oldValue) {
				return 1;
			} else if (newValue < oldValue) {
				return -1;
			} else {
				return 0;
			}
		}
	});
	
})();
